﻿// © ANO PO "KSI", 2016

#pragma once
#include "qtcommon.h"

#include <memory>

enum class DiffReason {
	HASH_DIFF,
	NEW_FILE,
	ABSENT_FILE,
	IO_ERROR
};

#include <hashlibpp.h>

struct HashFile { 
	std::string filename, hash; 
	HashFile() {}
	HashFile(const std::string& _filename, const std::string& _hash) : filename(_filename), hash(_hash) {}
};
using HashFileList = std::vector<HashFile>;

class CompareErr;
using CompareErrList = std::vector<std::unique_ptr<CompareErr>>;

class ICreateHashReport {
	
public:	
	virtual void start(int totalFiles) = 0;
	virtual void complete(/*Status _status*/)=0;
	virtual void setCurrentFile(const QString& filename) = 0;
	virtual void setError(const QString& errDescr) = 0;

	virtual bool wasCancelled() const = 0;
};

class HashTools : public QObject {
	Q_OBJECT

	hashwrapper *m_pHashWrapper;
	std::string m_hashType;

public:
	HashTools(const std::string& hashType, QObject * parent = nullptr);
	~HashTools();

public:
	void createHashList(const QString& dirName, HashFileList* pHashFileList, ICreateHashReport* report = nullptr);
	void updateHashList(HashFileList* pHashFileList);
	std::string loadDB(const std::string& dbname/*, const QDir& controlDir*/, HashFileList* pHashFileList);
	std::string createDB(const std::string& dbname, const HashFileList& hashFileList);
	std::string hashType() const;
	void compare(const QDir& baseDir, const HashFileList& newHashFileList, const HashFileList& dbHashFileList, /*std::vector<std::unique_ptr<CompareErr>>*/ CompareErrList* errors);

private:	
	void createHashList(const QString& dirName, const QDir& baseDir, HashFileList* pHashFileList, ICreateHashReport* report);
	void createFileList(const QDir& dir, QFileInfoList* fileList);
};

class CompareErr : public QObject {
	Q_OBJECT
protected:
 	DiffReason m_reason;
	QString m_filename;
	void setDiffReason(DiffReason _reason);
public:
	CompareErr(const std::string& _filename, DiffReason _reason);

	const QString& filename() const { return m_filename; }

	virtual QString text();
	virtual QString descrText() { return QString(); }
 };

class CE_HashDiff : public CompareErr {
	Q_OBJECT
private:
	QString m_newhash, m_dbhash;
public:
	CE_HashDiff(const std::string& _filename, const std::string& _newhash, const std::string& _dbhash);
	virtual QString descrText() override;
	virtual QString hash() const { return QString(); }
};

class CE_NewFile : public CompareErr{
	Q_OBJECT
public:
	CE_NewFile(const std::string& _filename);
	virtual QString descrText() override;
};

class CE_AbsentFile : public CompareErr {	
	Q_OBJECT
public:
	CE_AbsentFile(const std::string& _filename);
	virtual QString descrText() override;
};

class CE_IOError : public CompareErr {
	Q_OBJECT

	QString m_msg;
public:
	CE_IOError(const std::string& _filename, std::string& _msg);
	virtual QString descrText() override;
};