//  ANO PO "KSI", 2016

#include "MainWindow.h"

#include "CreationDBResult.h"
#include "CheckDBResult.h"
#include "CreateHashWindow.h"

MainWindow::MainWindow(QWidget *parent)
	: QDialog(parent),
	m_pHashTools(std::make_unique<HashTools>("sha256", this)),
	m_pSettings(new QSettings(QSettings::IniFormat, QSettings::SystemScope, "Ksi", "KsiHashCheck", this))
{
	QPushButton* pButtonCreateDB = new QPushButton(style()->standardIcon(QStyle::SP_FileIcon),tr("Create checksums.."));
	QPushButton* pButtonCheckDB = new QPushButton(style()->standardIcon(QStyle::SP_FileDialogContentsView),tr("Verify checksums.."));
	QPushButton* pButtonAbout = new QPushButton(style()->standardIcon(QStyle::SP_FileDialogInfoView),tr("About.."));
	QPushButton* pButtonExit = new QPushButton(style()->standardIcon(QStyle::SP_DialogCancelButton), tr("Exit"));

	QGridLayout* pCmdLayout = new QGridLayout;
	pCmdLayout->addWidget(pButtonCreateDB,0,0);
	pCmdLayout->addWidget(pButtonCheckDB,1,0);
	pCmdLayout->addWidget(pButtonAbout,0,1);
	pCmdLayout->addWidget(pButtonExit,1,1);
	
	m_pCreateDBWindow = new CreateDBWindow(this);
	m_pCreateDBWindow->setSettings(m_pSettings);
	m_pCheckDBWindow = new CheckDBWindow(this);
	m_pCheckDBWindow->setSettings(m_pSettings);
// 	m_pCreateHashWindow = new CreateHashWindow(this);
// 	m_pCreateHashWindow->setSettings(m_pSettings);
	
	setLayout(pCmdLayout);

	connect(pButtonCreateDB, &QPushButton::clicked, m_pCreateDBWindow, &CreateDBWindow::exec);
	connect(pButtonCheckDB, &QPushButton::clicked, m_pCheckDBWindow, &CheckDBWindow::exec);

	connect(m_pCreateDBWindow, &CreateDBWindow::accepted, this, &MainWindow::createDB);
	connect(m_pCheckDBWindow, &CheckDBWindow::accepted, this, &MainWindow::checkDB);
	connect(pButtonAbout, &QPushButton::clicked, this, &MainWindow::showAbout);
	connect(pButtonExit, &QPushButton::clicked, this, &QDialog::close);
}

MainWindow::~MainWindow()
{

}

bool MainWindow::createHashList(const QString& dirName, HashFileList* hfl)
{
	CreateHashWindow hashWindow;

	m_pHashTools->createHashList(dirName, hfl, &hashWindow);

	return hashWindow.successfullyompleted();
}

void MainWindow::createDB()
{
	auto dirname = m_pCreateDBWindow->dir();
	auto dbfilename = m_pCreateDBWindow->dbfile();

	if (!QDir(dirname).exists()) {
		QMessageBox::warning(this, tr("Error"), tr("Directory is not exists"));
		return;
	}

	if (dbfilename.length() == 0) {
		QMessageBox::warning(this, tr("Error"), tr("File name is not specified"));
		return;
	}

	QFileInfo dirInfo(dirname);
	
	HashFileList hfl;

	if (!createHashList(dirname, &hfl)) {
		return;
	}

	auto dbhash = m_pHashTools->createDB(dbfilename.toLocal8Bit().toStdString(), hfl);
	
	if (dbhash.length() == 0) {
		QMessageBox::warning(this, tr("Error"), tr("Checksums file creation error"));
	}
	else	{
		CreationDBResult dlg(dbfilename, QString::fromStdString(dbhash), this);
		dlg.exec();
	}	
}

void MainWindow::checkDB()
{
	auto dirname = m_pCheckDBWindow->dir();
	auto dbfilename = m_pCheckDBWindow->dbfile();

	if (!QDir(dirname).exists()) {
		QMessageBox::warning(this, tr("Error"), tr("Directory is not exists"));
		return;
	}

	QFileInfo dbinfo = QFileInfo(dbfilename);

	if (!QFile::exists(dbfilename)) {
		QMessageBox::warning(this, tr("Error"), tr("File name is not specified"));
		return;
	}

	HashFileList hfl, dbhfl;
	CompareErrList cmperr;
	

	if (!createHashList(dirname, &hfl)) {
		return;
	}

	auto dbhash = m_pHashTools->loadDB(dbfilename.toLocal8Bit().toStdString(), &dbhfl);

	if (dbhash.length() == 0) {
		QMessageBox::warning(this, tr("Error"), tr("Checksums file load error"));
		return;
	}

	m_pHashTools->compare(dirname, hfl, dbhfl, &cmperr);

	CheckDBResult checkDBResult(dbfilename, QString::fromStdString(dbhash),cmperr);

	checkDBResult.exec();
}

void MainWindow::showAbout()
{
	QMessageBox::information(this, tr("About"), QString::fromLatin1("KsiHashCheck v1.0\n\n ANO PO \"KSI\", 2016\n\n hashlib++. Benjamin Grdelbach, 2007-2010\n Qt. The Qt Company Ltd, 2015"));
}
